/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include "oirSupport.h"
#include "fitsio.h"
#include "interpUtil.h"
#include "imaging_fdata.h"

/**********************************************************
 * Dedistort IMAGING_FDATA file using polynomial in imaging_detector
 * Usage:
 *    Dedistort  infile outfile
 *        infile  is an IMAGING_FDATA tables
 *        outfile is IMAGING_FDATA table with result.
 ************************************************************* */

int main (int argc, char** argv) {
   char *infile, *outfile;
   int i, ireg, ierr;
   long nrow, irow;
   char outFileName[130];
   char *bang="!";
   fitsfile *inFile = NULL;
   fitsfile *outFile = NULL;
   imaging_fdata_table_def *idata=NULL, *odata=NULL;
   imaging_detector_table_def *detect;
   imaging_fdata_row *inData=NULL;

   ierr = 0;
   /* Set inputs */
   infile = argv[1];
   outfile = argv[2];

   /*  Open input  */
   i = fits_open_file(&inFile, infile, READONLY, &ierr) ;
   if (ierr != 0) {
      fprintf (stderr, "error opening input file %s\n", infile);
      fits_report_error(stderr, ierr);
      return 1;
   }

   /* Get detector section */
   detect = read_imaging_detector_table(inFile, &ierr);
   if (ierr != 0) {
     fprintf(stderr, "error %i opening detector table in %s\n", 
	     ierr, infile);
     fits_report_error(stderr, ierr);
     return 1;
   }
   
   /*  Open input table */
   idata = open_imaging_fdata_table(inFile, &ierr);
   if (ierr !=0) {
     fprintf(stderr,"error %i opening imaging data in file %s\n", 
            ierr, infile);
     fits_report_error(stderr, ierr);
     return 1;
   }
   /* get size of table */
   fits_get_num_rows (inFile, &nrow, &ierr);


   /* Prepare output file - add '!' to allow cfitsio to overwrite */
   strcpy(outFileName, bang);
   strcat(outFileName, outfile);

   fits_create_file (&outFile, outFileName, &ierr);
   if (ierr !=0) {
     fprintf (stderr, "error %d opening fits output %s\n",
	      ierr, outfile);
     fits_report_error(stderr, ierr);
     return 1;
   }
   
   odata = make_imaging_fdata_table_def (outFile, detect, 1, &ierr);
   if (ierr !=0) {
      fprintf (stderr, "error creating data definition\n");
      return 1;
   }

   /*  create row */
   inData = make_imaging_fdata_row(idata);

   /* Loop over input file, reading, dedistorting and writing */
   for (irow=0; irow<nrow; irow++) {
     read_imaging_fdata_table(inFile, idata, inData, irow, &ierr);
     if (ierr !=0) {
       fprintf(stderr, "error %i reading imaging data row %i in file %s\n", 
	       ierr, irow, infile);
       fits_report_error(stderr, ierr);
       return 1;
     }
     
     /* dedistort */
     for (ireg=0; ireg<detect->nregion; ireg++) 
       minrtsDedistortFrame (detect, ireg, inData, inData);

     /* write to output */
     write_imaging_fdata_table(outFile, odata, inData, irow, &ierr);
     if (ierr !=0) {
       fprintf(stderr, "error %i writing imaging data row %i in file %s\n", 
	       ierr, irow, outfile);
       fits_report_error(stderr, ierr);
       return 1;
     }
  } /* end loop over input file */


   /*  close */
   close_imaging_fdata_table(inFile, idata, &ierr);
   close_imaging_fdata_table(outFile, odata, &ierr);
   if (ierr !=0) {
     fits_report_error(stderr, ierr);
     return 1;
   }

   /* Cleanup */
   kill_imaging_fdata_row(inData);
   
   return 0;
} /* end main */

